<?php

namespace Drupal\decoupled_cookie_auth\EventSubscriber;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\Core\Routing\CurrentRouteMatch;
use Drupal\Core\Session\AccountProxyInterface;
use Symfony\Component\HttpKernel\Event\ResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Class ModifiedResourceResponseSubscriber subscribes to kernel.response.
 */
class ModifiedResourceResponseSubscriber implements EventSubscriberInterface {

  /**
   * Drupal\Core\Routing\CurrentRouteMatch definition.
   *
   * @var \Drupal\Core\Routing\CurrentRouteMatch
   */
  protected $currentRouteMatch;

  /**
   * Drupal\Core\Session\AccountProxyInterface definition.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The JSON serializer.
   *
   * @var \Drupal\Component\Serialization\Json
   */
  protected $jsonSerializer;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new ModifiedResourceResponseSubscriber object.
   */
  public function __construct(CurrentRouteMatch $current_route_match, AccountProxyInterface $current_user, Json $json_serializer, ConfigFactoryInterface $config_factory, EntityTypeManagerInterface $entity_type_manager) {
    $this->currentRouteMatch = $current_route_match;
    $this->currentUser = $current_user;
    $this->jsonSerializer = $json_serializer;
    $this->configFactory = $config_factory;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[KernelEvents::RESPONSE] = ['handleResponse'];

    return $events;
  }

  /**
   * This method is called when the kernel.response is dispatched.
   *
   * Automatically log in a user when they register a new account.
   *
   * @param \Symfony\Component\HttpKernel\Event\ResponseEvent $event
   *   The dispatched event.
   */
  public function handleResponse(ResponseEvent $event) {
    $response = $event->getResponse();
    $is_route = $this->currentRouteMatch->getRouteName() === 'rest.user_registration.POST';
    if ($is_route && $this->currentUser->isAnonymous()) {
      $data = $this->jsonSerializer->decode($response->getContent());
      /** @var \Drupal\user\UserInterface $account */
      $account = $this->entityTypeManager->getStorage('user')->load($data);
      $config = $this->configFactory->get('user.settings');
      if (!$config->get('verify_mail') && !empty($account) && $account->isActive()) {
        user_login_finalize($account);
      }
    }
  }

}
